from pathlib import Path
from hatchling.metadata.plugin.interface import MetadataHookInterface
from pymodaq_utils.resources.hatch_build_plugins import update_metadata_from_toml

here = Path(__file__).absolute().parent


class PluginInfoTomlHook(MetadataHookInterface):
    def update(self, metadata: dict) -> None:
        update_metadata_from_toml(metadata, here)



from pymodaq.utils.data import DataFromPlugins, DataToExport
from pymodaq.control_modules.viewer_utility_classes import DAQ_Viewer_base, comon_parameters
from pymodaq.utils.parameter import Parameter
from pymodaq_plugins_cellkraft.hardware.cellkraft.Eseries import CellKraftE1500Drivers

from pymodaq_plugins_cellkraft import config
from pymodaq.utils.logger import set_logger, get_module_name

logger = set_logger(get_module_name(__file__))


class DAQ_0DViewer_Pressure(DAQ_Viewer_base):
    """

        pymodaq Version 5.0.11
        pymodaq-data Version 5.0.23

        Limites thérorique a ne pas dépasser :
            - Flow : 2 g/min                                                                - modifiable     - move
           X- Pressure                                                                      - non modifiable - viewer0D
            - Steam Temperature : 180 °C                                                    - modifiable     - move
            - Tube Temperature : 200 °C                                                     - modifiable     - move
            - RH (Relative Humidity) : 80%                                                  - modifiable     - move

        """

    params = comon_parameters + [
        {
            'title': 'Device:',
            'name': 'device',
            'type': 'str',
            'value': 'Cellkraft E1500 Series',
            'readonly': True
        },
        {
            'title': 'Host:',
            'name': 'host',
            'type': 'str',
            'value': config('Cellkraft', 'DEVICE01', 'host')
        },
        {
            'title': 'Info:',
            'name': 'info',
            'type': 'str',
            'value': 'Nothing',
            'readonly': True
        }]

    desc = {'Pressure': "Lecture en 0.01 près"}

    def ini_attributes(self):
        self.controller: CellKraftE1500Drivers
        self.settings.child('info').setValue(self.desc['Pressure'])
        pass

    def commit_settings(self, param: Parameter):
        """Apply the consequences of a change of value in the detector settings

        Parameters
        ----------
        param: Parameter
            A given parameter (within detector_settings) whose value has been changed by the user
        """
        pass

    def ini_detector(self, controller=None):
        """Actuator communication initialization

        Parameters
        ----------
        controller: (object)
            custom object of a PyMoDAQ plugin (Slave case). None if only one actuator by controller (Master case)

        Returns
        -------
        info: str
        initialized: bool
            False if initialization failed otherwise True
        """

        if self.is_master:  # Master Case : controller == None
            controller = CellKraftE1500Drivers(self.settings['host'])  # Create control
            self.controller = controller
            initialized = self.controller.init_hardware()  # Init connection
            info = "Initialized in Master"

        else:  # Slave Case : controller != None
            self.controller = self.ini_detector_init(slave_controller=controller)
            initialized = self.controller.instr.connected
            info = "Initialized in Slave"

        return info, initialized

    def close(self):
        """Terminate the communication protocol"""
        self.controller.close()
        logger.info("Communication ended successfully")

    def grab_data(self, Naverage=1, live=False, **kwargs):
        """Start a grab from the detector

        Parameters
        ----------
        Naverage: int
            Number of hardware averaging (if hardware averaging is possible, self.hardware_averaging should be set to
            True in class preamble, and you should code this implementation)
        kwargs: dict
            others optionals arguments
        """
        data_tot = self.controller.Get_Pressure()
        self.dte_signal.emit(
            DataToExport(name='Graph',
                         data=[DataFromPlugins(
                            name='Pressure',
                            data=data_tot,
                            dim='Data0D',
                            labels=['Pressure in bar'])
                            ]
                         )
            )

    def stop(self):
        """Stop the current grab hardware wise if necessary"""
        return ''



from pymodaq.utils.logger import set_logger, get_module_name
from pymodaq_plugins_cellkraft.hardware.tcpmodbus import SyncModBusInstrument
from enum import IntEnum
logger = set_logger(get_module_name(__file__))
# WRITE
#
# pump control : register 9107 value [0 auto 1 manual 2 prime] default 1
# SP temp var °C : register 9300 value [] default 10
# RH% : register 9240 default 10
# SP flow(g/min) register 9310 default 10
# SP TEMP tube °C register 9355
# Pump % 9109
#
# READ
#
# steam temp 4148
# Air humidity 4628
# flow (g/min) 6518
# pressure(bar) 5268
# Tube temp 4468
# pump % 6158
#
# turn down => write 0 to register 9310
# read status 6518


class Pump(IntEnum):
    read_address = 6158
    write_address = 9109
    default_write_value = 100
    scaling = 100
    read_scaling = scaling
    write_scaling = scaling


class PumpMode(IntEnum):
    write_address = 9107
    mode_auto = 0
    mode_manual = 1
    mode_prime = 2
    default_mode = mode_auto
    default_write_value = default_mode


class Steam(IntEnum):
    read_address = 4148
    write_address = 9300
    default_write_value = 0
    read_scaling = 10
    write_scaling = 1


class Air(IntEnum):
    read_address = 4628
    write_address = 9240
    default_write_value = 105.0
    scaling = 10
    read_scaling = scaling
    write_scaling = scaling


class Flow(IntEnum):
    read_address = 6518
    write_address = 9310
    default_write_value = 105.0
    scaling = 10
    read_scaling = scaling
    write_scaling = scaling


class Tube(IntEnum):
    read_address = 4468
    write_address = 9355
    default_write_value = 105.0
    read_scaling = 10
    write_scaling = 10


class Pressure(IntEnum):
    read_address = 5268
    read_scaling = 100


Eseries_Config = {
        1500: {
            "general": {
                "scaling_default": 1,
                },
            PumpMode.__name__: {
                "reference": PumpMode,
                "authorized_write_value": [0, 1, 2],
                },
            Pump.__name__: {
                "reference": Pump,
                "unit": "%",
                "type": float,
                "authorized_write_value": range(0, 105, 1),
                },
            Steam.__name__: {
                "reference": Steam,
                "type": float,
                "unit": "C",
                "authorized_write_value": range(0, 200, 1),
                },
            Air.__name__: {
                "reference": Air,
                "unit": "%",
                "type": float,
                "authorized_write_value": range(0, 105, 1),
                },
            Flow.__name__: {
                "reference": Flow,
                "unit": "g/min",
                "type": float,
                "authorized_write_value": [value for value in range(0, 250, 1)],
                },
            Tube.__name__: {
                "reference": Tube,
                "unit": "C",
                "type": float,
                "authorized_write_value": range(0, 200, 1),
                },
            Pressure.__name__: {
                "reference": Pressure,
                "unit": "Bar",
                "type": float,
                # "write_address": 9355,
                # "authorized_write_value": range(0, 200, 1),
                # "default_write_value": 105
                },
            }
        }


class CellKraftE1500Drivers:
    """TCP ModBus driver for the Steam Generator CellKraft E-series

    Relies on a custom tcpmodules based on pymodbus (source : https://github.com/pymodbus-dev/pymodbus
    documentation : https://pymodbus.readthedocs.io/en/latest/)
    """
    def __init__(self, host, config=None):
        """Initialize the Steam Generator driver

        :param host: hostname or ip address
        """
        self.instr = SyncModBusInstrument(host)
        self.host = host
        self.registers = {}
        self.init = False

        if config is None:
            self.config = Eseries_Config
        else:
            self.config = config

        self.ini_register(self.config)

    def ini_register(self, config_dict=None):
        """
        Initialise the register to expose the method/hardware parameters
        :param config_dict: Manual Configuration dictionary feeding should be self.config
        """

        if config_dict is None:
            if self.config is None:
                config_dict = Eseries_Config
            else:
                config_dict = self.config

        self.registers["PumpSetMode"] = {
            "method": self.PumpSetMode,
            "reference": config_dict[1500]["PumpMode"]["reference"],
            "register": config_dict[1500]["PumpMode"]["reference"].write_address.value,
            "mode": "write"
        }
        self.registers["Pump"] = {
            "method": self.Write_Pump,
            "reference": config_dict[1500]["Pump"]["reference"],
            "write_register": config_dict[1500]["Pump"]["reference"].write_address.value,
            "read_register": config_dict[1500]["Pump"]["reference"].read_address.value,
            "mode": "write",
            "scaling": config_dict[1500]["Pump"]["reference"].scaling.value
        }
        self.registers["SP_SteamT"] = {
            "method": self.SP_SteamT,
            "reference": config_dict[1500]["Steam"]["reference"],
            "register": config_dict[1500]["Steam"]["reference"].write_address.value,
            "mode": "write",
            "scaling": config_dict[1500]["Steam"]["reference"].write_scaling.value
        }
        self.registers["RH"] = {
            "method": self.RH,
            "reference": config_dict[1500]["Air"]["reference"],
            "register": config_dict[1500]["Air"]["reference"].write_address.value,
            "mode": "write",
            "scaling": config_dict[1500]["Air"]["reference"].scaling.value
        }
        self.registers["SP_Flow"] = {
            "method": self.SP_Flow,
            "reference": config_dict[1500]["Flow"]["reference"],
            "register": config_dict[1500]["Flow"]["reference"].write_address.value,
            "mode": "write",
            "scaling": config_dict[1500]["Flow"]["reference"].scaling.value
        }
        self.registers["SP_Tube_Temp"] = {
            "method": self.SP_Tube_Temp,
            "reference": config_dict[1500]["Tube"]["reference"],
            "register": config_dict[1500]["Tube"]["reference"].write_address.value,
            "mode": "write",
            "scaling": config_dict[1500]["Tube"]["reference"].write_scaling.value
        }
        self.registers["Get_Steam_T"] = {
            "method": self.Get_Steam_T,
            "reference": config_dict[1500]["Steam"]["reference"],
            "register": config_dict[1500]["Steam"]["reference"].read_address.value,
            "mode": "read",
            "scaling": config_dict[1500]["Steam"]["reference"].read_scaling.value
        }
        self.registers["Get_Air_H"] = {
            "method": self.Get_Air_H,
            "reference": config_dict[1500]["Air"]["reference"],
            "register": config_dict[1500]["Air"]["reference"].read_address.value,
            "mode": "read",
            "scaling": config_dict[1500]["Air"]["reference"].read_scaling.value
        }
        self.registers["Get_Flow"] = {
            "method": self.Get_Flow,
            "reference": config_dict[1500]["Flow"]["reference"],
            "register": config_dict[1500]["Flow"]["reference"].read_address.value,
            "mode": "read",
            "scaling": config_dict[1500]["Flow"]["reference"].read_scaling.value
        }
        self.registers["Get_Pressure"] = {
            "method": self.Get_Pressure,
            "reference": config_dict[1500]["Pressure"]["reference"],
            "register": config_dict[1500]["Pressure"]["reference"].read_address.value,
            "mode": "read",
            "scaling": config_dict[1500]["Pressure"]["reference"].read_scaling.value
        }
        self.registers["Get_Tube_T"] = {
            "method": self.Get_Tube_T,
            "reference": config_dict[1500]["Tube"]["reference"],
            "register": config_dict[1500]["Tube"]["reference"].read_address.value,
            "mode": "read",
            "scaling": config_dict[1500]["Tube"]["reference"].read_scaling.value
        }

    def init_hardware(self):
        """Connect and initialize the Steam Generator

        """
        self.init = self.instr.ini_hw()
        return self.init

    def stop(self):
        """Stop procedure

        :return:
        """
        self.SP_Flow(0)

    def close(self):
        """Close connection

        :return:
        """
        self.instr.close()

# ------------------------------------------------------ #

    def PumpSetMode(self, value: str = "auto"):
        """Writing the pump mode

        Auto : La pompe fonctionne automatiquement,
        sûrement en fonction des besoins du système.

        Manual : La pompe fonctionne en mode manuel,
        l'utilisateur doit contrôler son fonctionnement manuellement.

        Prime : Ce mode est généralement utilisé pour amorcer la pompe,
        pour remplir le système avec du liquide avant de commencer le fonctionnement normal.

        :param value: human-readable equivalent of the 3 allowed values (0 auto, 1 manual, 2 prime) defaulting to auto
        """

        order: int
        match value:
            case "auto":
                order = PumpMode.mode_auto.value
            case "manual":
                order = PumpMode.mode_manual.value
            case "prime":
                order = PumpMode.mode_prime.value
            case _:
                order = PumpMode.default_mode.value
        try:
            self.instr.write(self.registers["PumpSetMode"]["register"],
                             order)
        except Exception as e:
            raise (Exception, f"error in {self.__qualname__}, {e}")

    def Write_Pump(self, pump_power: int = 100):
        """Set the pump power in %

        :param pump_power: int% pump power
        """
        pump_power = int(pump_power*self.registers["Pump"]["scaling"])
        try:
            self.instr.write(self.registers["Pump"]["write_register"], pump_power)
        except Exception as e:
            raise (Exception, f"error {e}")
        return pump_power

    def Read_Pump(self):
        """Get the pump power in %

        :return pump_power: int% pump power
        """
        pump_power = self.instr.read(self.registers["Pump"]["read_register"])
        if isinstance(pump_power, Exception):
            raise pump_power
        else:
            return pump_power.registers[0] / self.registers["Pump"]["scaling"]

# ------------------------------------------------------ #

    def SP_SteamT(self, temperature: int | float = 10):
        """Set the SP Steam temperature in °C

        :param temperature: int in °C
        :return:
        """
        temperature = int(temperature * self.registers["SP_SteamT"]["scaling"])  # Scaling => 1
        try:
            self.instr.write(self.registers["SP_SteamT"]["register"], temperature)
        except Exception as e:
            raise (Exception, f"error {e}")
        return temperature

    def RH(self, relativehumidity: int | float = 105):
        """Set the relative humidity in %

        :param relativehumidity: int% relative humidity
        """
        relativehumidity = int(relativehumidity * self.registers["RH"]["scaling"])  # Scaling => 10
        try:
            self.instr.write(self.registers["RH"]["register"], relativehumidity)
        except Exception as e:
            raise (Exception, f"error {e}")
        return relativehumidity

    def SP_Flow(self, flow: int | float):
        """Set the flow in g/min

        :param flow:
        """
        flow = int(flow*self.registers["SP_Flow"]["scaling"])  # Scaling => 10
        try:
            self.instr.write(self.registers["SP_Flow"]["register"], flow)
        except Exception as e:
            raise (Exception, f"error {e}")
        return flow

    def SP_Tube_Temp(self, temperature: int | float):
        """Set the tube temperature

        :param temperature: tube temperature set point
        :return:
        """
        temperature = int(temperature * self.registers["SP_Tube_Temp"]["scaling"])  # Scaling => 10
        try:
            self.instr.write(self.registers["SP_Tube_Temp"]["register"], temperature)
        except Exception as e:
            raise (Exception, f"error {e}")
        return temperature

# ------------------------------------------------------ #

    def Get_Steam_T(self):
        """Get the steam temperature

        :return: temperature int °C
        """

        readresult = self.instr.read(self.registers["Get_Steam_T"]["register"])
        if isinstance(readresult, Exception):
            raise readresult
        else:
            return readresult.registers[0]/self.registers["Get_Steam_T"]["scaling"]

    def Get_Air_H(self):
        """Get the air humidity

        :return: int %
        """

        readresult = self.instr.read(self.registers["Get_Air_H"]["register"])
        if isinstance(readresult, Exception):
            raise readresult
        else:
            return readresult.registers[0]/self.registers["Get_Air_H"]["scaling"]

    def Get_Flow(self):
        """Get the air humidity

        :return: int %
        """

        readresult = self.instr.read(self.registers["Get_Flow"]["register"])
        if isinstance(readresult, Exception):
            raise readresult
        else:
            return readresult.registers[0]/self.registers["Get_Flow"]["scaling"]

    def Get_Pressure(self):
        """Get the pressure

        :return: int Bar
        """

        readresult = self.instr.read(self.registers["Get_Pressure"]["register"])
        if isinstance(readresult, Exception):
            raise readresult
        else:
            return readresult.registers[0]/self.registers["Get_Pressure"]["scaling"]

    def Get_Tube_T(self):
        """Get the tube temperature

        :return: int °C
        """

        readresult = self.instr.read(self.registers["Get_Tube_T"]["register"])
        if isinstance(readresult, Exception):
            raise readresult
        else:
            return readresult.registers[0]/self.registers["Get_Tube_T"]["scaling"]


from pathlib import Path
from pymodaq.utils.logger import set_logger  # to be imported by other modules.

from .utils import Config
config = Config()

with open(str(Path(__file__).parent.joinpath('resources/VERSION')), 'r') as fvers:
    __version__ = fvers.read().strip()



import numpy as np
from pymodaq.utils.daq_utils import ThreadCommand
from pymodaq.utils.data import DataFromPlugins, Axis, DataToExport
from pymodaq.control_modules.viewer_utility_classes import DAQ_Viewer_base, comon_parameters, main
from pymodaq.utils.parameter import Parameter


class PythonWrapperOfYourInstrument:
    #  TODO Replace this fake class with the import of the real python wrapper of your instrument
    pass

# TODO:
# (1) change the name of the following class to DAQ_1DViewer_TheNameOfYourChoice
# (2) change the name of this file to daq_1Dviewer_TheNameOfYourChoice ("TheNameOfYourChoice" should be the SAME
#     for the class name and the file name.)
# (3) this file should then be put into the right folder, namely IN THE FOLDER OF THE PLUGIN YOU ARE DEVELOPING:
#     pymodaq_plugins_my_plugin/daq_viewer_plugins/plugins_1D
class DAQ_1DViewer_Template(DAQ_Viewer_base):
    """ Instrument plugin class for a 1D viewer.
    
    This object inherits all functionalities to communicate with PyMoDAQ’s DAQ_Viewer module through inheritance via
    DAQ_Viewer_base. It makes a bridge between the DAQ_Viewer module and the Python wrapper of a particular instrument.

    TODO Complete the docstring of your plugin with:
        * The set of instruments that should be compatible with this instrument plugin.
        * With which instrument it has actually been tested.
        * The version of PyMoDAQ during the test.
        * The version of the operating system.
        * Installation instructions: what manufacturer’s drivers should be installed to make it run?

    Attributes:
    -----------
    controller: object
        The particular object that allow the communication with the hardware, in general a python wrapper around the
         hardware library.
         
    # TODO add your particular attributes here if any

    """
    params = comon_parameters+[
        ## TODO for your custom plugin
        # elements to be added here as dicts in order to control your custom stage
        ############
        ]

    def ini_attributes(self):
        #  TODO declare the type of the wrapper (and assign it to self.controller) you're going to use for easy
        #  autocompletion
        self.controller: PythonWrapperOfYourInstrument = None

        # TODO declare here attributes you want/need to init with a default value

        self.x_axis = None

    def commit_settings(self, param: Parameter):
        """Apply the consequences of a change of value in the detector settings

        Parameters
        ----------
        param: Parameter
            A given parameter (within detector_settings) whose value has been changed by the user
        """
        ## TODO for your custom plugin
        if param.name() == "a_parameter_you've_added_in_self.params":
           self.controller.your_method_to_apply_this_param_change()
#        elif ...
        ##

    def ini_detector(self, controller=None):
        """Detector communication initialization

        Parameters
        ----------
        controller: (object)
            custom object of a PyMoDAQ plugin (Slave case). None if only one actuator/detector by controller
            (Master case)

        Returns
        -------
        info: str
        initialized: bool
            False if initialization failed otherwise True
        """

        raise NotImplemented  # TODO when writing your own plugin remove this line and modify the one below
        self.ini_detector_init(slave_controller=controller)

        if self.is_master:
            self.controller = PythonWrapperOfYourInstrument()  #instantiate you driver with whatever arguments are needed
            self.controller.open_communication() # call eventual methods

        ## TODO for your custom plugin
        # get the x_axis (you may want to to this also in the commit settings if x_axis may have changed
        data_x_axis = self.controller.your_method_to_get_the_x_axis()  # if possible
        self.x_axis = Axis(data=data_x_axis, label='', units='', index=0)

        # TODO for your custom plugin. Initialize viewers pannel with the future type of data
        self.dte_signal_temp.emit(DataToExport(name='myplugin',
                                               data=[DataFromPlugins(name='Mock1',
                                                                     data=[np.array([0., 0., ...]),
                                                                           np.array([0., 0., ...])],
                                                                     dim='Data1D', labels=['Mock1', 'label2'],
                                                                     axes=[self.x_axis])]))

        info = "Whatever info you want to log"
        initialized = True
        return info, initialized

    def close(self):
        """Terminate the communication protocol"""
        ## TODO for your custom plugin
        raise NotImplemented  # when writing your own plugin remove this line
        #  self.controller.your_method_to_terminate_the_communication()  # when writing your own plugin replace this line

    def grab_data(self, Naverage=1, **kwargs):
        """Start a grab from the detector

        Parameters
        ----------
        Naverage: int
            Number of hardware averaging (if hardware averaging is possible, self.hardware_averaging should be set to
            True in class preamble and you should code this implementation)
        kwargs: dict
            others optionals arguments
        """
        ## TODO for your custom plugin: you should choose EITHER the synchrone or the asynchrone version following

        ##synchrone version (blocking function)
        data_tot = self.controller.your_method_to_start_a_grab_snap()
        self.dte_signal.emit(DataToExport('myplugin',
                                          data=[DataFromPlugins(name='Mock1', data=data_tot,
                                                                dim='Data1D', labels=['dat0', 'data1'],
                                                                axes=[self.x_axis])]))

        ##asynchrone version (non-blocking function with callback)
        self.controller.your_method_to_start_a_grab_snap(self.callback)
        #########################################################


    def callback(self):
        """optional asynchrone method called when the detector has finished its acquisition of data"""
        data_tot = self.controller.your_method_to_get_data_from_buffer()
        self.dte_signal.emit(DataToExport('myplugin',
                                          data=[DataFromPlugins(name='Mock1', data=data_tot,
                                                                dim='Data1D', labels=['dat0', 'data1'])]))

    def stop(self):
        """Stop the current grab hardware wise if necessary"""
        ## TODO for your custom plugin
        raise NotImplemented  # when writing your own plugin remove this line
        self.controller.your_method_to_stop_acquisition()  # when writing your own plugin replace this line
        self.emit_status(ThreadCommand('Update_Status', ['Some info you want to log']))
        ##############################
        return ''


if __name__ == '__main__':
    main(__file__)



from typing import Union, List, Dict
from pymodaq_plugins_cellkraft import config
from pymodaq.control_modules.move_utility_classes import DAQ_Move_base, comon_parameters_fun, DataActuatorType,\
    DataActuator  # common set of parameters for all actuators
from pymodaq.utils.daq_utils import ThreadCommand  # object used to send info back to the main thread
from pymodaq.utils.parameter import Parameter
from pymodaq_plugins_cellkraft.hardware.cellkraft.Eseries import CellKraftE1500Drivers
from pymodaq_utils.logger import set_logger, get_module_name

logger = set_logger(get_module_name(__file__))


class DAQ_Move_CellkraftE1500(DAQ_Move_base):
    """

    pymodaq Version 5.0.11
    pymodaq-data Version 5.0.23

    Limites thérorique a ne pas dépasser :
            - Flow : 2 g/min                                                                - modifiable     - move
           X- Pressure                                                                      - non modifiable - viewer0D
            - Steam Temperature : 180 °C                                                    - modifiable     - move
            - Tube Temperature : 200 °C                                                     - modifiable     - move
            - RH (Relative Humidity) : 80%                                                  - modifiable     - move

    - Modifiez le fichier : ...\site-packages\pymodaq\control_modules\daq_move.py
    - A la ligne  615, modifiez :
    - self.ui.set_unit_as_suffix(self.get_unit_to_display(unit))
    - en
    - self.ui.set_unit_as_suffix(unit)
    - Cela empeche la conversion des unites rentrée dans _controller_units

    - Si des "m" sont devant les unitées dans le dashboard, copier cela dans le config_pymodaq.toml:
    [actuator]
    epsilon_default = 1
    polling_interval_ms = 100
    polling_timeout_s = 20  # s
    refresh_timeout_ms = 500  # ms
    siprefix = false
    siprefix_even_without_units = false
    display_units = true

    """
    is_multiaxes = False

    _axis_names: Union[List[str], Dict[str, str]] = ['Flow', 'Steam_Temperature', 'Tube_Temperature', 'RH', 'Pressure']
    _controller_units: Union[str, List[str]] = ['g/min', '°C', '°C', '%', 'bar']
    _epsilon: Union[float, List[float]] = [0.1, 0.1, 0.1, 1, 1]

    data_actuator_type = DataActuatorType.DataActuator

    params = [{'title': 'Device:', 'name': 'device', 'type': 'str', 'value': 'Cellkraft E1500 Series',
               'readonly': True},
              {'title': 'Host:', 'name': 'host', 'type': 'str', 'value': config('Cellkraft', 'DEVICE01', 'host')},
              {'title': 'Write Limit:', 'name': 'limit', 'type': 'str', 'value': 'Nothing'},
              {'title': 'Info:', 'name': 'info', 'type': 'str', 'value': 'Nothing', 'readonly': True},
              ] + comon_parameters_fun(is_multiaxes=is_multiaxes, axis_names=_axis_names, epsilon=_epsilon)

    # Information sur chaque actuateurs
    desc = {'Flow': "Lecture en 0.1 près, Ecriture en 0.1 près\n"
                    "En cas de dépassement de cette limite, "
                    "le surplus sera effacé (exemple : 1.25 va ressortir en 1.2)",

            'Steam_Temperature': "Lecture en 0.1 près, Ecriture en 1 près\n"
                    "En cas de dépassement de cette limite, "
                    "le surplus sera effacé (exemple : 100.1 va ressortir en 100)",

            'Tube_Temperature': "Lecture en 0.1 près, Ecriture en 0.1 près\n"
                    "En cas de dépassement de cette limite, "
                    "le surplus sera effacé (exemple : 100.25 va ressortir en 100.2)",

            'RH': "Lecture en 0.1 près, Ecriture en 0.1 près\n"
                    "En cas de dépassement de cette limite, "
                    "le surplus sera effacé (exemple : 95.25 va ressortir en 95.2)",
            'Pressure': "None"
            }

    # Limite de valeur pour chaque actuateur, (modifiable dans le dashboard via le show settings)
    lim = {'Flow': 2.5,
           'Steam_Temperature': 160,
           'Tube_Temperature': 160,
           'RH': 100,
           'Pressure': 0
           }

    current_axes: str

    def ini_attributes(self):
        self.controller: CellKraftE1500Drivers
        self.current_axes = self.settings.child('multiaxes', 'axis').value()
        self.change_param()
        pass

    def change_param(self):

        if self.current_axes == 'Flow':
            self.settings.child('info').setValue(self.desc['Flow'])
            self.settings.child('limit').setValue(self.lim['Flow'])

        elif self.current_axes == 'Steam_Temperature':
            self.settings.child('info').setValue(self.desc['Steam_Temperature'])
            self.settings.child('limit').setValue(self.lim['Steam_Temperature'])

        elif self.current_axes == 'Tube_Temperature':
            self.settings.child('info').setValue(self.desc['Tube_Temperature'])
            self.settings.child('limit').setValue(self.lim['Tube_Temperature'])

        elif self.current_axes == 'RH':
            self.settings.child('info').setValue(self.desc['RH'])
            self.settings.child('limit').setValue(self.lim['RH'])

        elif self.current_axes == 'Pressure':
            self.settings.child('info').setValue(self.desc['Pressure'])
            self.settings.child('limit').setValue(self.lim['Pressure'])

    def get_actuator_value(self):
        """Get the current value from the hardware with scaling conversion.

        Returns
        -------
        float: The position obtained after scaling conversion.
        """
        if self.current_axes == 'Flow':
            flow = DataActuator(data=self.controller.Get_Flow())
            flow = self.get_position_with_scaling(flow)
            return flow

        elif self.current_axes == 'RH':
            air_h = DataActuator(data=self.controller.Get_Air_H())
            air_h = self.get_position_with_scaling(air_h)
            return air_h

        elif self.current_axes == 'Steam_Temperature':
            steam_t = DataActuator(data=self.controller.Get_Steam_T())
            steam_t = self.get_position_with_scaling(steam_t)
            return steam_t

        elif self.current_axes == 'Tube_Temperature':
            tube_t = DataActuator(data=self.controller.Get_Tube_T())
            tube_t = self.get_position_with_scaling(tube_t)
            return tube_t

        elif self.current_axes == 'Pressure':
            pressure = DataActuator(data=self.controller.Get_Pressure())
            pressure = self.get_position_with_scaling(pressure)
            return pressure

        else:
            self.emit_status(ThreadCommand('Update_Status',
                                           ['WARNING - No Axis Selected, self.current_axes can be None']))

    def user_condition_to_reach_target(self) -> bool:
        """ Implement a condition for exiting the polling mechanism and specifying that the
        target value has been reached

       Returns
        -------
        bool: if True, PyMoDAQ considers the target value has been reached
        """
        #  either delete this method if the usual polling is fine with you, but if need you can
        #  add here some other condition to be fullfilled either a completely new one or
        #  using or/and operations between the epsilon_bool and some other custom booleans
        #  for a usage example see DAQ_Move_brushlessMotor from the Thorlabs plugin
        return True

    def close(self):
        """Terminate the communication protocol"""
        self.controller.close()

    def commit_settings(self, param: Parameter):
        """Apply the consequences of a change of value in the detector settings

        Parameters
        ----------
        param: Parameter
            A given parameter (within detector_settings) whose value has been changed by the user
        """
        if param.name() == 'axis':
            if param.value() == 'Flow':
                self.axis_unit = self._controller_units[0]
                self.settings.child('units').value = self._controller_units[0]
                self.current_axes = 'Flow'

            elif param.value() == 'Steam_Temperature':
                self.axis_unit = self._controller_units[1]
                self.settings.child('units').value = self._controller_units[1]
                self.current_axes = 'Steam_Temperature'

            elif param.value() == 'Tube_Temperature':
                self.axis_unit = self._controller_units[2]
                self.settings.child('units').value = self._controller_units[2]
                self.current_axes = 'Tube_Temperature'

            elif param.value() == 'RH':
                self.axis_unit = self._controller_units[3]
                self.settings.child('units').value = self._controller_units[3]
                self.current_axes = 'RH'

            elif param.value() == 'Pressure':
                self.axis_unit = self._controller_units[4]
                self.settings.child('units').value = self._controller_units[4]
                self.current_axes = 'Pressure'
        pass

    def ini_stage(self, controller=None):
        """Actuator communication initialization

        Parameters
        ----------
        controller: (object)
            custom object of a PyMoDAQ plugin (Slave case). None if only one actuator by controller (Master case)

        Returns
        -------
        info: str
        initialized: bool
            False if initialization failed otherwise True
        """

        # Debug Master/Slave
        # if controller is None:
        #     self.emit_status(ThreadCommand('Update_Status', [f'Controller is None - {self.current_axes} is Master']))
        # else:
        #   self.emit_status(ThreadCommand('Update_Status', [f'Controller is not None - {self.current_axes} is Slave']))

        if self.is_master:  # Master Case : controller == None
            controller = CellKraftE1500Drivers(self.settings['host'])  # Create control
            self.controller = controller
            initialized = self.controller.init_hardware()  # Init connection
            info = "Initialized in Master"

        else:  # Slave Case : controller != None
            self.controller = self.ini_stage_init(slave_controller=controller)
            initialized = self.controller.instr.connected
            info = "Initialized in Slave"
        self.controller.PumpSetMode("auto")
        return info, initialized

    def move_abs(self, value: DataActuator):
        """ Move the actuator to the absolute target defined by value

        Parameters
        ----------
        value: (float) value of the absolute target positioning
        """
        value = self.check_bound(value)  # if user checked bounds, the defined bounds are applied here
        self.target_value = value
        value = self.set_position_with_scaling(value)  # apply scaling if the user specified one

        self.move_value(value)

    def move_rel(self, value: DataActuator):
        """ Move the actuator to the relative target actuator value defined by value

        Parameters
        ----------
        value: (float) value of the relative target positioning
        """
        value = self.check_bound(self.current_position + value) - self.current_position
        self.target_value = value + self.current_position
        value = self.set_position_relative_with_scaling(value)

        self.move_value(value)

    def move_value(self, value):
        """ Move the actuator to the target actuator value defined by value
        Used by move_rel() and move_abs()
        Parameters
        ----------
        value: (float) value of the target positioning
        """
        limit = self.settings.child('limit').opts['value']
        if self.current_axes == 'Flow':
            if not value.value() < float(limit):
                self.emit_status(ThreadCommand('Update_Status', [f'WARNING - Flow have to be < {limit}']))
            else:
                val = self.controller.SP_Flow(value.value())*0.1
                self.emit_status(ThreadCommand('Update_Status', [f'Flow set to {val}']))

        elif self.current_axes == 'RH':
            if not value.value() < float(limit):
                self.emit_status(ThreadCommand('Update_Status', [f'WARNING - RH have to be < {limit}']))
            else:
                val = self.controller.RH(value.value())*0.1
                self.emit_status(ThreadCommand('Update_Status', [f'RH set to {val}']))

        elif self.current_axes == 'Steam_Temperature':
            if not value.value() < float(limit):
                self.emit_status(ThreadCommand('Update_Status', [f'WARNING - Steam_Temp have to be < {limit}']))
            else:
                val = self.controller.SP_SteamT(int(value.value()))
                self.emit_status(ThreadCommand('Update_Status', [f'Steam_Temp set to {val}']))

        elif self.current_axes == 'Tube_Temperature':
            if not value.value() < float(limit):
                self.emit_status(ThreadCommand('Update_Status', [f'WARNING - Tube_Temp have to be < {limit}']))
            else:
                val = self.controller.SP_Tube_Temp(int(value.value()))*0.1
                self.emit_status(ThreadCommand('Update_Status', [f'Tube_Temp set to {val}']))

        elif self.current_axes == 'Pressure':
            self.emit_status(ThreadCommand('Update_Status', ["WARNING - Can't modify the pressure"]))

        else:
            self.emit_status(ThreadCommand('Update_Status',
                                           ['WARNING - Nothing moved - Problem with current_axes variable in'
                                            'daq_move_CELLkraftE1500.py - WARNING']))

    def move_home(self, value: Union[float, DataActuator]):
        """
        Set the value to 0 / can change this value later
        """
        pass

    def stop_motion(self):
        """
        Stop the actuator and emits move_done signal
        Pump is the only one that can be stopped, the other are just values that we change
        """
        pass



from qtpy import QtWidgets

from pymodaq.utils import gui_utils as gutils
from pymodaq.utils.config import Config
from pymodaq.utils.logger import set_logger, get_module_name

# todo: replace here *pymodaq_plugins_modbus* by your plugin package name
from pymodaq_plugins_template.utils import Config as PluginConfig

logger = set_logger(get_module_name(__file__))

main_config = Config()
plugin_config = PluginConfig()


# todo: modify the name of this class to reflect its application and change the name in the main
# method at the end of the script
class CustomAppTemplate(gutils.CustomApp):

    # todo: if you wish to create custom Parameter and corresponding widgets. These will be
    # automatically added as children of self.settings. Morevover, the self.settings_tree will
    # render the widgets in a Qtree. If you wish to see it in your app, add is into a Dock
    params = []

    def __init__(self, parent: gutils.DockArea):
        super().__init__(parent)

        self.setup_ui()

    def setup_docks(self):
        """Mandatory method to be subclassed to setup the docks layout

        Examples
        --------
        >>>self.docks['ADock'] = gutils.Dock('ADock name')
        >>>self.dockarea.addDock(self.docks['ADock'])
        >>>self.docks['AnotherDock'] = gutils.Dock('AnotherDock name')
        >>>self.dockarea.addDock(self.docks['AnotherDock'''], 'bottom', self.docks['ADock'])

        See Also
        --------
        pyqtgraph.dockarea.Dock
        """
        # todo: create docks and add them here to hold your widgets
        # reminder, the attribute self.settings_tree will  render the widgets in a Qtree.
        # If you wish to see it in your app, add is into a Dock
        raise NotImplementedError

    def setup_actions(self):
        """Method where to create actions to be subclassed. Mandatory

        Examples
        --------
        >>> self.add_action('quit', 'Quit', 'close2', "Quit program")
        >>> self.add_action('grab', 'Grab', 'camera', "Grab from camera", checkable=True)
        >>> self.add_action('load', 'Load', 'Open', "Load target file (.h5, .png, .jpg) or data from camera"
            , checkable=False)
        >>> self.add_action('save', 'Save', 'SaveAs', "Save current data", checkable=False)

        See Also
        --------
        ActionManager.add_action
        """
        raise NotImplementedError(f'You have to define actions here')

    def connect_things(self):
        """Connect actions and/or other widgets signal to methods"""
        raise NotImplementedError

    def setup_menu(self):
        """Non mandatory method to be subclassed in order to create a menubar

        create menu for actions contained into the self._actions, for instance:

        Examples
        --------
        >>>file_menu = self.mainwindow.menuBar().addMenu('File')
        >>>self.affect_to('load', file_menu)
        >>>self.affect_to('save', file_menu)

        >>>file_menu.addSeparator()
        >>>self.affect_to('quit', file_menu)

        See Also
        --------
        pymodaq.utils.managers.action_manager.ActionManager
        """
        # todo create and populate menu using actions defined above in self.setup_actions
        pass

    def value_changed(self, param):
        """ Actions to perform when one of the param's value in self.settings is changed from the
        user interface

        For instance:
        if param.name() == 'do_something':
            if param.value():
                print('Do something')
                self.settings.child('main_settings', 'something_done').setValue(False)

        Parameters
        ----------
        param: (Parameter) the parameter whose value just changed
        """
        pass


def main():
    from pymodaq.utils.gui_utils.utils import mkQApp
    app = mkQApp('CustomApp')

    mainwindow = QtWidgets.QMainWindow()
    dockarea = gutils.DockArea()
    mainwindow.setCentralWidget(dockarea)

    # todo: change the name here to be the same as your app class
    prog = CustomAppTemplate(dockarea)

    mainwindow.show()

    app.exec()


if __name__ == '__main__':
    main()



from pymodbus.client import ModbusTcpClient
from pymodaq.utils.logger import set_logger, get_module_name
logger = set_logger(get_module_name(__file__))


class SyncModBusInstrument:
    """

    """
    def __init__(self, host, port = 502):
        self.connected = False
        self.host = host
        self.port = port
        self.precision = 1
        self.modbus = ModbusTcpClient(self.host)
        self.registerdict = {}

    def close(self):
        """End the connection
        """
        self.modbus.close()
        self.connected = False

    def write(self, register, value):
        """

        :param register:
        :param value:
        :return:
        """

        self.modbus.write_register(register, value)

    def read(self, register):
        """

        :param register:
        :return:
        """
        return self.modbus.read_input_registers(register)

    def ini_hw(self):
        """

        """
        try:
            self.modbus.connect()
            self.connected = True
            return self.connected
        except:
            return False

    def addregister(self, name, address, values, readwrite):
        """

        :param adress:
        :param values:
        :param readwrite:
        :return:
        """
        if str(address) not in address.keys():
            self.registerdict[str(address)]= {'authorizedvalues' :values, 'rwstatus': readwrite}